def convert_signed_byte_to_int(byte):
   if int(byte) < 127:
       return int(byte)
   else:
       return -1 * (255 - int(byte) + 1)


def packetize(data):
    """
    input:(list) data=[ id (string),
                        pm25 (int),
                        pm10 (int), 
                        temp (float), 
                        humidity (float), 
                        rtc (4 hex values)]
    return: (list) frame = [ID (2-bytes),
                            PM2.5:(2-bytes),
                            PM10:(2-bytes),
                            TEMP:(1-byte),
                            HUM:(1-byte),
                            RTC:(4-byte)]
    +----+-------+------+------+-----+-----+
    | ID | PM2.5 | PM10 | TEMP | HUM | RTC |
    +----+-------+------+------+-----+-----+
    """
    try:
    	assert len(data) == 6
    	assert len(data[0]) == 2
    	assert all(list(map(lambda i : i <= 65535, data[1:3])))
    	assert all(list(map(lambda i : i >= -128 and i <= 127, data[3:5])))
    except AssertionError:
        # do somthing
        return -1

    frame = b''
    # encoding (id)
    for char in data[0]: # id
        frame += ord(char).to_bytes(1, 'big')
    
    # encoding PMXX fields
    frame += data[1].to_bytes(2, 'big')
    frame += data[2].to_bytes(2, 'big')

    frame += int(data[3]).to_bytes(length=1, byteorder='big', signed=True)
    frame += int(data[4]).to_bytes(1, 'big')
    
    rtc_value = data[5]
    frame += rtc_value.to_bytes(4, 'big')
    return frame



def depacketize(frame):
    try:
        len(frame) == 12
        type(frame) == bytes
    except AssertionError:
        # do something
        return -1

    # decoding id
    id = ''
    for byte in frame[0:2]:
        id += chr(byte)
    
    pm25 = int(frame[2:4].hex(), 16)
    pm10 = int(frame[4:6].hex(), 16)
    
    temp = float(convert_signed_byte_to_int(frame[6]))
    hum  = float(int(frame[7]))

    rtc = hex(int.from_bytes(frame[8:], 'big'))

    return [id, pm25, pm10, temp, hum, rtc]


if __name__=='__main__':
    id = 'B1'
    pm25 = 213
    pm10 = 3255
    temp = -27.0
    humidity = 10.0
    rtc = 0x2342

    data = [id, pm25, pm10, temp, humidity, rtc]
    frame = packetize(data)

    print("frame to be sent: {}".format(frame))
    
    
    rx_data = depacketize(frame)
    print("data to be recieved: {}".format(rx_data))
